@@ -7,7 +7,11 @@
 use panic_halt as _;
 
 use cortex_m_rt::entry;
-use stm32f1xx_hal::{pac, prelude::*, pwm_input::*, timer::Timer};
+use stm32f1xx_hal::{
+    pac,
+    prelude::*,
+    timer::{pwm_input::*, Timer},
+};
 
 #[entry]
 fn main() -> ! {
@@ -27,11 +31,11 @@ fn main() -> ! {
     let (_pa15, _pb3, pb4) = afio.mapr.disable_jtag(gpioa.pa15, gpiob.pb3, gpiob.pb4);
     let pb5 = gpiob.pb5;
 
-    let pwm_input = Timer::tim3(p.TIM3, &clocks).pwm_input(
+    let pwm_input = Timer::new(p.TIM3, &clocks).pwm_input(
         (pb4, pb5),
         &mut afio.mapr,
         &mut dbg,
-        Configuration::Frequency(10.khz()),
+        Configuration::Frequency(10.kHz()),
     );
 
     loop {
