@@ -1,4 +1,5 @@
 #![deny(unsafe_code)]
+#![allow(clippy::empty_loop)]
 #![no_std]
 #![no_main]
 
@@ -44,7 +45,7 @@ fn setup() -> (
         (sck, miso, mosi),
         &mut afio.mapr,
         MODE,
-        1_u32.mhz(),
+        1.MHz(),
         clocks,
     );
 
