@@ -6,20 +6,14 @@ use embedded_hal::adc::{Channel, OneShot};
 #[cfg(all(feature = "stm32f103", any(feature = "high", feature = "xl",),))]
 use crate::dma::dma2;
 use crate::dma::{dma1, CircBuffer, Receive, RxDma, Transfer, TransferPayload, W};
-#[cfg(all(feature = "stm32f103", any(feature = "high", feature = "xl",),))]
-use crate::gpio::gpiof;
-use crate::gpio::Analog;
-use crate::gpio::{gpioa, gpiob, gpioc};
+use crate::gpio::{self, Analog};
 use crate::rcc::{Clocks, Enable, Reset};
+use crate::time::kHz;
 use core::sync::atomic::{self, Ordering};
 use cortex_m::asm::delay;
-use embedded_dma::StaticWriteBuffer;
+use embedded_dma::WriteBuffer;
 
-#[cfg(feature = "stm32f103")]
-use crate::pac::ADC2;
-#[cfg(all(feature = "stm32f103", any(feature = "high", feature = "xl")))]
-use crate::pac::ADC3;
-use crate::pac::{ADC1, RCC};
+use crate::pac::{self, RCC};
 
 /// Continuous mode
 pub struct Continuous;
@@ -107,7 +101,7 @@ impl From<Align> for bool {
 }
 
 macro_rules! adc_pins {
-    ($ADC:ident, $($pin:ty => $chan:expr),+ $(,)*) => {
+    ($ADC:ty, $($pin:ty => $chan:expr),+ $(,)*) => {
         $(
             impl Channel<$ADC> for $pin {
                 type ID = u8;
@@ -118,60 +112,60 @@ macro_rules! adc_pins {
     };
 }
 
-adc_pins!(ADC1,
-    gpioa::PA0<Analog> => 0_u8,
-    gpioa::PA1<Analog> => 1_u8,
-    gpioa::PA2<Analog> => 2_u8,
-    gpioa::PA3<Analog> => 3_u8,
-    gpioa::PA4<Analog> => 4_u8,
-    gpioa::PA5<Analog> => 5_u8,
-    gpioa::PA6<Analog> => 6_u8,
-    gpioa::PA7<Analog> => 7_u8,
-    gpiob::PB0<Analog> => 8_u8,
-    gpiob::PB1<Analog> => 9_u8,
-    gpioc::PC0<Analog> => 10_u8,
-    gpioc::PC1<Analog> => 11_u8,
-    gpioc::PC2<Analog> => 12_u8,
-    gpioc::PC3<Analog> => 13_u8,
-    gpioc::PC4<Analog> => 14_u8,
-    gpioc::PC5<Analog> => 15_u8,
+adc_pins!(pac::ADC1,
+    gpio::PA0<Analog> => 0_u8,
+    gpio::PA1<Analog> => 1_u8,
+    gpio::PA2<Analog> => 2_u8,
+    gpio::PA3<Analog> => 3_u8,
+    gpio::PA4<Analog> => 4_u8,
+    gpio::PA5<Analog> => 5_u8,
+    gpio::PA6<Analog> => 6_u8,
+    gpio::PA7<Analog> => 7_u8,
+    gpio::PB0<Analog> => 8_u8,
+    gpio::PB1<Analog> => 9_u8,
+    gpio::PC0<Analog> => 10_u8,
+    gpio::PC1<Analog> => 11_u8,
+    gpio::PC2<Analog> => 12_u8,
+    gpio::PC3<Analog> => 13_u8,
+    gpio::PC4<Analog> => 14_u8,
+    gpio::PC5<Analog> => 15_u8,
 );
 
-#[cfg(any(feature = "stm32f103",))]
-adc_pins!(ADC2,
-    gpioa::PA0<Analog> => 0_u8,
-    gpioa::PA1<Analog> => 1_u8,
-    gpioa::PA2<Analog> => 2_u8,
-    gpioa::PA3<Analog> => 3_u8,
-    gpioa::PA4<Analog> => 4_u8,
-    gpioa::PA5<Analog> => 5_u8,
-    gpioa::PA6<Analog> => 6_u8,
-    gpioa::PA7<Analog> => 7_u8,
-    gpiob::PB0<Analog> => 8_u8,
-    gpiob::PB1<Analog> => 9_u8,
-    gpioc::PC0<Analog> => 10_u8,
-    gpioc::PC1<Analog> => 11_u8,
-    gpioc::PC2<Analog> => 12_u8,
-    gpioc::PC3<Analog> => 13_u8,
-    gpioc::PC4<Analog> => 14_u8,
-    gpioc::PC5<Analog> => 15_u8,
+#[cfg(any(feature = "stm32f103", feature = "connectivity"))]
+adc_pins!(pac::ADC2,
+    gpio::PA0<Analog> => 0_u8,
+    gpio::PA1<Analog> => 1_u8,
+    gpio::PA2<Analog> => 2_u8,
+    gpio::PA3<Analog> => 3_u8,
+    gpio::PA4<Analog> => 4_u8,
+    gpio::PA5<Analog> => 5_u8,
+    gpio::PA6<Analog> => 6_u8,
+    gpio::PA7<Analog> => 7_u8,
+    gpio::PB0<Analog> => 8_u8,
+    gpio::PB1<Analog> => 9_u8,
+    gpio::PC0<Analog> => 10_u8,
+    gpio::PC1<Analog> => 11_u8,
+    gpio::PC2<Analog> => 12_u8,
+    gpio::PC3<Analog> => 13_u8,
+    gpio::PC4<Analog> => 14_u8,
+    gpio::PC5<Analog> => 15_u8,
 );
 
 #[cfg(all(feature = "stm32f103", any(feature = "high", feature = "xl",),))]
-adc_pins!(ADC3,
-    gpioa::PA0<Analog> => 0_u8,
-    gpioa::PA1<Analog> => 1_u8,
-    gpioa::PA2<Analog> => 2_u8,
-    gpioa::PA3<Analog> => 3_u8,
-    gpiof::PF6<Analog> => 4_u8,
-    gpiof::PF7<Analog> => 5_u8,
-    gpiof::PF8<Analog> => 6_u8,
-    gpiof::PF9<Analog> => 7_u8,
-    gpiof::PF10<Analog> => 8_u8,
-    gpioc::PC0<Analog> => 10_u8,
-    gpioc::PC1<Analog> => 11_u8,
-    gpioc::PC2<Analog> => 12_u8,
-    gpioc::PC3<Analog> => 13_u8,
+adc_pins!(pac::ADC3,
+    gpio::PA0<Analog> => 0_u8,
+    gpio::PA1<Analog> => 1_u8,
+    gpio::PA2<Analog> => 2_u8,
+    gpio::PA3<Analog> => 3_u8,
+    gpio::PF6<Analog> => 4_u8,
+    gpio::PF7<Analog> => 5_u8,
+    gpio::PF8<Analog> => 6_u8,
+    gpio::PF9<Analog> => 7_u8,
+    gpio::PF10<Analog> => 8_u8,
+    gpio::PC0<Analog> => 10_u8,
+    gpio::PC1<Analog> => 11_u8,
+    gpio::PC2<Analog> => 12_u8,
+    gpio::PC3<Analog> => 13_u8,
 );
 
 /// Stored ADC config can be restored using the `Adc::restore_cfg` method
@@ -180,7 +174,7 @@ pub struct StoredConfig(SampleTime, Align);
 
 macro_rules! adc_hal {
     ($(
-        $ADC:ident: ($adc:ident),
+        $ADC:ty: ($adc:ident),
     )+) => {
         $(
 
@@ -205,9 +199,9 @@ macro_rules! adc_hal {
                     // The manual states that we need to wait two ADC clocks cycles after power-up
                     // before starting calibration, we already delayed in the power-up process, but
                     // if the adc clock is too low that was not enough.
-                    if s.clocks.adcclk().0 < 2_500_000 {
-                        let two_adc_cycles = s.clocks.sysclk().0 / s.clocks.adcclk().0 *2;
-                        let already_delayed = s.clocks.sysclk().0 / 800_000;
+                    if s.clocks.adcclk() < kHz(2500) {
+                        let two_adc_cycles = s.clocks.sysclk() / s.clocks.adcclk() * 2;
+                        let already_delayed = s.clocks.sysclk() / kHz(800);
                         if two_adc_cycles > already_delayed {
                             delay(two_adc_cycles - already_delayed);
                         }
@@ -269,7 +263,7 @@ macro_rules! adc_hal {
                     // this time can be found in the datasheets.
                     // Here we are delaying for approximately 1us, considering 1.25 instructions per
                     // cycle. Do we support a chip which needs more than 1us ?
-                    delay(self.clocks.sysclk().0 / 800_000);
+                    delay(self.clocks.sysclk() / kHz(800));
                 }
 
                 fn power_down(&mut self) {
@@ -278,17 +272,17 @@ macro_rules! adc_hal {
 
                 fn reset(&mut self) {
                     let rcc = unsafe { &(*RCC::ptr()) };
-                    $ADC::reset(rcc);
+                    <$ADC>::reset(rcc);
                 }
 
                 fn enable_clock(&mut self) {
                     let rcc = unsafe { &(*RCC::ptr()) };
-                    $ADC::enable(rcc);
+                    <$ADC>::enable(rcc);
                 }
 
                 fn disable_clock(&mut self) {
                     let rcc = unsafe { &(*RCC::ptr()) };
-                    $ADC::disable(rcc);
+                    <$ADC>::disable(rcc);
                 }
 
                 fn calibrate(&mut self) {
@@ -460,7 +454,7 @@ macro_rules! adc_hal {
     }
 }
 
-impl Adc<ADC1> {
+impl Adc<pac::ADC1> {
     fn read_aux(&mut self, chan: u8) -> u16 {
         let tsv_off = if self.rb.cr2.read().tsvrefe().bit_is_clear() {
             self.rb.cr2.modify(|_, w| w.tsvrefe().set_bit());
@@ -469,7 +463,7 @@ impl Adc<ADC1> {
             // sensor, this time can be found in the datasheets.
             // Here we are delaying for approximately 10us, considering 1.25 instructions per
             // cycle. Do we support a chip which needs more than 10us ?
-            delay(self.clocks.sysclk().0 / 80_000);
+            delay(self.clocks.sysclk().raw() / 80_000);
             true
         } else {
             false
@@ -509,7 +503,7 @@ impl Adc<ADC1> {
         // recommended ADC sampling for temperature sensor is 17.1 usec,
         // so use the following approximate settings
         // to support all ADC frequencies
-        let sample_time = match self.clocks.adcclk().0 {
+        let sample_time = match self.clocks.adcclk().raw() {
             0..=1_200_000 => SampleTime::T_1,
             1_200_001..=1_500_000 => SampleTime::T_7,
             1_500_001..=2_400_000 => SampleTime::T_13,
@@ -544,17 +538,17 @@ impl Adc<ADC1> {
 }
 
 adc_hal! {
-    ADC1: (adc1),
+    pac::ADC1: (adc1),
 }
 
-#[cfg(feature = "stm32f103")]
+#[cfg(any(feature = "stm32f103", feature = "connectivity"))]
 adc_hal! {
-    ADC2: (adc2),
+    pac::ADC2: (adc2),
 }
 
 #[cfg(all(feature = "stm32f103", any(feature = "high", feature = "xl",),))]
 adc_hal! {
-    ADC3: (adc3),
+    pac::ADC3: (adc3),
 }
 
 pub struct AdcPayload<ADC, PINS, MODE> {
@@ -607,7 +601,7 @@ pub trait SetChannels<PINS>: ChannelTimeSequence {
 pub type AdcDma<ADC, PINS, MODE, CHANNEL> = RxDma<AdcPayload<ADC, PINS, MODE>, CHANNEL>;
 
 macro_rules! adcdma {
-    ($ADCX:ident: (
+    ($ADCX:ty: (
         $rxdma:ident,
         $dmarxch:ty,
     )) => {
@@ -741,15 +735,15 @@ macro_rules! adcdma {
         impl<B, PINS, MODE> crate::dma::CircReadDma<B, u16> for AdcDma<$ADCX, PINS, MODE, $dmarxch>
         where
             Self: TransferPayload,
-            &'static mut [B; 2]: StaticWriteBuffer<Word = u16>,
+            &'static mut [B; 2]: WriteBuffer<Word = u16>,
             B: 'static,
         {
             fn circ_read(mut self, mut buffer: &'static mut [B; 2]) -> CircBuffer<B, Self> {
                 // NOTE(unsafe) We own the buffer now and we won't call other `&mut` on it
                 // until the end of the transfer.
-                let (ptr, len) = unsafe { buffer.static_write_buffer() };
+                let (ptr, len) = unsafe { buffer.write_buffer() };
                 self.channel.set_peripheral_address(
-                    unsafe { &(*$ADCX::ptr()).dr as *const _ as u32 },
+                    unsafe { &(*<$ADCX>::ptr()).dr as *const _ as u32 },
                     false,
                 );
                 self.channel.set_memory_address(ptr as u32, true);
@@ -781,14 +775,14 @@ macro_rules! adcdma {
         impl<B, PINS, MODE> crate::dma::ReadDma<B, u16> for AdcDma<$ADCX, PINS, MODE, $dmarxch>
         where
             Self: TransferPayload,
-            B: StaticWriteBuffer<Word = u16>,
+            B: WriteBuffer<Word = u16>,
         {
             fn read(mut self, mut buffer: B) -> Transfer<W, B, Self> {
                 // NOTE(unsafe) We own the buffer now and we won't call other `&mut` on it
                 // until the end of the transfer.
-                let (ptr, len) = unsafe { buffer.static_write_buffer() };
+                let (ptr, len) = unsafe { buffer.write_buffer() };
                 self.channel.set_peripheral_address(
-                    unsafe { &(*$ADCX::ptr()).dr as *const _ as u32 },
+                    unsafe { &(*<$ADCX>::ptr()).dr as *const _ as u32 },
                     false,
                 );
                 self.channel.set_memory_address(ptr as u32, true);
@@ -818,7 +812,7 @@ macro_rules! adcdma {
 }
 
 adcdma! {
-    ADC1: (
+    pac::ADC1: (
         AdcDma1,
         dma1::C1,
     )
@@ -826,7 +820,7 @@ adcdma! {
 
 #[cfg(all(feature = "stm32f103", any(feature = "high", feature = "xl",),))]
 adcdma! {
-    ADC3: (
+    pac::ADC3: (
         AdcDma3,
         dma2::C5,
     )
