@@ -189,6 +189,7 @@ use sealed::PinMode;
 
 use sealed::Interruptable;
 impl<MODE> Interruptable for Input<MODE> {}
+impl Interruptable for Dynamic {}
 
 /// External Interrupt Pin
 pub trait ExtiPin {
@@ -404,6 +405,8 @@ macro_rules! gpio {
                 }
             }
         }
+
+        pub use $gpiox::{ $($PXi,)+ };
     }
 }
 
