@@ -37,25 +37,19 @@ use core::ops::Deref;
 use core::ptr;
 
 pub use crate::hal::spi::{FullDuplex, Mode, Phase, Polarity};
-#[cfg(any(feature = "high", feature = "connectivity"))]
-use crate::pac::SPI3;
-use crate::pac::{RCC, SPI1, SPI2};
+use crate::pac::{self, RCC};
 
 use crate::afio::MAPR;
 use crate::dma::dma1;
 #[cfg(feature = "connectivity")]
 use crate::dma::dma2;
 use crate::dma::{Receive, RxDma, RxTxDma, Transfer, TransferPayload, Transmit, TxDma, R, W};
-use crate::gpio::gpioa::{PA5, PA6, PA7};
-use crate::gpio::gpiob::{PB13, PB14, PB15, PB3, PB4, PB5};
-#[cfg(feature = "connectivity")]
-use crate::gpio::gpioc::{PC10, PC11, PC12};
-use crate::gpio::{Alternate, Input};
-use crate::rcc::{Clocks, Enable, GetBusFreq, Reset};
+use crate::gpio::{self, Alternate, Input};
+use crate::rcc::{BusClock, Clocks, Enable, Reset};
 use crate::time::Hertz;
 
 use core::sync::atomic::{self, Ordering};
-use embedded_dma::{StaticReadBuffer, StaticWriteBuffer};
+use embedded_dma::{ReadBuffer, WriteBuffer};
 
 /// SPI error
 #[derive(Debug)]
@@ -121,37 +115,37 @@ impl<REMAP> Miso<REMAP> for NoMiso {}
 impl<REMAP> Mosi<REMAP> for NoMosi {}
 
 macro_rules! remap {
-    ($name:ident, $SPIX:ident, $state:literal, $SCK:ident, $MISO:ident, $MOSI:ident) => {
+    ($name:ident, $SPIX:ty, $state:literal, $SCK:ident, $MISO:ident, $MOSI:ident) => {
         pub struct $name;
         impl Remap for $name {
             type Periph = $SPIX;
             const REMAP: bool = $state;
         }
-        impl<MODE> Sck<$name> for $SCK<Alternate<MODE>> {}
-        impl<MODE> Miso<$name> for $MISO<Input<MODE>> {}
-        impl<MODE> Mosi<$name> for $MOSI<Alternate<MODE>> {}
+        impl<MODE> Sck<$name> for gpio::$SCK<Alternate<MODE>> {}
+        impl<MODE> Miso<$name> for gpio::$MISO<Input<MODE>> {}
+        impl<MODE> Mosi<$name> for gpio::$MOSI<Alternate<MODE>> {}
     };
 }
 
-remap!(Spi1NoRemap, SPI1, false, PA5, PA6, PA7);
-remap!(Spi1Remap, SPI1, true, PB3, PB4, PB5);
-remap!(Spi2NoRemap, SPI2, false, PB13, PB14, PB15);
+remap!(Spi1NoRemap, pac::SPI1, false, PA5, PA6, PA7);
+remap!(Spi1Remap, pac::SPI1, true, PB3, PB4, PB5);
+remap!(Spi2NoRemap, pac::SPI2, false, PB13, PB14, PB15);
 #[cfg(any(feature = "high", feature = "connectivity"))]
-remap!(Spi3NoRemap, SPI3, false, PB3, PB4, PB5);
+remap!(Spi3NoRemap, pac::SPI3, false, PB3, PB4, PB5);
 #[cfg(feature = "connectivity")]
-remap!(Spi3Remap, SPI3, true, PC10, PC11, PC12);
+remap!(Spi3Remap, pac::SPI3, true, PC10, PC11, PC12);
 
 pub trait Instance:
-    crate::Sealed + Deref<Target = crate::pac::spi1::RegisterBlock> + Enable + Reset + GetBusFreq
+    crate::Sealed + Deref<Target = crate::pac::spi1::RegisterBlock> + Enable + Reset + BusClock
 {
 }
 
-impl Instance for SPI1 {}
-impl Instance for SPI2 {}
+impl Instance for pac::SPI1 {}
+impl Instance for pac::SPI2 {}
 #[cfg(any(feature = "high", feature = "connectivity"))]
-impl Instance for SPI3 {}
+impl Instance for pac::SPI3 {}
 
-impl<REMAP, PINS> Spi<SPI1, REMAP, PINS, u8> {
+impl<REMAP, PINS> Spi<pac::SPI1, REMAP, PINS, u8> {
     /**
       Constructs an SPI instance using SPI1 in 8bit dataframe mode.
 
@@ -159,25 +153,24 @@ impl<REMAP, PINS> Spi<SPI1, REMAP, PINS, u8> {
 
       You can also use `NoSck`, `NoMiso` or `NoMosi` if you don't want to use the pins
     */
-    pub fn spi1<F>(
-        spi: SPI1,
+    pub fn spi1(
+        spi: pac::SPI1,
         pins: PINS,
         mapr: &mut MAPR,
         mode: Mode,
-        freq: F,
+        freq: Hertz,
         clocks: Clocks,
     ) -> Self
     where
-        F: Into<Hertz>,
-        REMAP: Remap<Periph = SPI1>,
+        REMAP: Remap<Periph = pac::SPI1>,
         PINS: Pins<REMAP>,
     {
         mapr.modify_mapr(|_, w| w.spi1_remap().bit(REMAP::REMAP));
-        Spi::<SPI1, _, _, u8>::_spi(spi, pins, mode, freq.into(), clocks)
+        Spi::<pac::SPI1, _, _, u8>::configure(spi, pins, mode, freq, clocks)
     }
 }
 
-impl<REMAP, PINS> Spi<SPI2, REMAP, PINS, u8> {
+impl<REMAP, PINS> Spi<pac::SPI2, REMAP, PINS, u8> {
     /**
       Constructs an SPI instance using SPI2 in 8bit dataframe mode.
 
@@ -185,18 +178,17 @@ impl<REMAP, PINS> Spi<SPI2, REMAP, PINS, u8> {
 
       You can also use `NoSck`, `NoMiso` or `NoMosi` if you don't want to use the pins
     */
-    pub fn spi2<F>(spi: SPI2, pins: PINS, mode: Mode, freq: F, clocks: Clocks) -> Self
+    pub fn spi2(spi: pac::SPI2, pins: PINS, mode: Mode, freq: Hertz, clocks: Clocks) -> Self
     where
-        F: Into<Hertz>,
-        REMAP: Remap<Periph = SPI2>,
+        REMAP: Remap<Periph = pac::SPI2>,
         PINS: Pins<REMAP>,
     {
-        Spi::<SPI2, _, _, u8>::_spi(spi, pins, mode, freq.into(), clocks)
+        Spi::<pac::SPI2, _, _, u8>::configure(spi, pins, mode, freq, clocks)
     }
 }
 
 #[cfg(any(feature = "high", feature = "connectivity"))]
-impl<REMAP, PINS> Spi<SPI3, REMAP, PINS, u8> {
+impl<REMAP, PINS> Spi<pac::SPI3, REMAP, PINS, u8> {
     /**
       Constructs an SPI instance using SPI3 in 8bit dataframe mode.
 
@@ -205,13 +197,12 @@ impl<REMAP, PINS> Spi<SPI3, REMAP, PINS, u8> {
       You can also use `NoSck`, `NoMiso` or `NoMosi` if you don't want to use the pins
     */
     #[cfg(not(feature = "connectivity"))]
-    pub fn spi3<F>(spi: SPI3, pins: PINS, mode: Mode, freq: F, clocks: Clocks) -> Self
+    pub fn spi3(spi: pac::SPI3, pins: PINS, mode: Mode, freq: Hertz, clocks: Clocks) -> Self
     where
-        F: Into<Hertz>,
-        REMAP: Remap<Periph = SPI3>,
+        REMAP: Remap<Periph = pac::SPI3>,
         PINS: Pins<REMAP>,
     {
-        Spi::<SPI3, _, _, u8>::_spi(spi, pins, mode, freq.into(), clocks)
+        Spi::<pac::SPI3, _, _, u8>::configure(spi, pins, mode, freq, clocks)
     }
 
     /**
@@ -222,21 +213,20 @@ impl<REMAP, PINS> Spi<SPI3, REMAP, PINS, u8> {
       You can also use `NoSck`, `NoMiso` or `NoMosi` if you don't want to use the pins
     */
     #[cfg(feature = "connectivity")]
-    pub fn spi3<F>(
-        spi: SPI3,
+    pub fn spi3(
+        spi: pac::SPI3,
         pins: PINS,
         mapr: &mut MAPR,
         mode: Mode,
-        freq: F,
+        freq: Hertz,
         clocks: Clocks,
     ) -> Self
     where
-        F: Into<Hertz>,
-        REMAP: Remap<Periph = SPI3>,
+        REMAP: Remap<Periph = pac::SPI3>,
         PINS: Pins<REMAP>,
     {
         mapr.modify_mapr(|_, w| w.spi3_remap().bit(REMAP::REMAP));
-        Spi::<SPI3, _, _, u8>::_spi(spi, pins, mode, freq.into(), clocks)
+        Spi::<pac::SPI3, _, _, u8>::configure(spi, pins, mode, freq, clocks)
     }
 }
 
@@ -333,7 +323,7 @@ impl<SPI, REMAP, PINS> Spi<SPI, REMAP, PINS, u8>
 where
     SPI: Instance,
 {
-    fn _spi(spi: SPI, pins: PINS, mode: Mode, freq: Hertz, clocks: Clocks) -> Self {
+    fn configure(spi: SPI, pins: PINS, mode: Mode, freq: Hertz, clocks: Clocks) -> Self {
         // enable or reset SPI
         let rcc = unsafe { &(*RCC::ptr()) };
         SPI::enable(rcc);
@@ -342,7 +332,7 @@ where
         // disable SS output
         spi.cr2.write(|w| w.ssoe().clear_bit());
 
-        let br = match SPI::get_frequency(&clocks).0 / freq.0 {
+        let br = match SPI::clock(&clocks) / freq {
             0 => unreachable!(),
             1..=2 => 0b000,
             3..=5 => 0b001,
@@ -517,7 +507,7 @@ pub type SpiRxTxDma<SPI, REMAP, PINS, RXCHANNEL, TXCHANNEL> =
     RxTxDma<Spi<SPI, REMAP, PINS, u8>, RXCHANNEL, TXCHANNEL>;
 
 macro_rules! spi_dma {
-    ($SPIi:ident, $RCi:ty, $TCi:ty, $rxdma:ident, $txdma:ident, $rxtxdma:ident) => {
+    ($SPIi:ty, $RCi:ty, $TCi:ty, $rxdma:ident, $txdma:ident, $rxtxdma:ident) => {
         pub type $rxdma<REMAP, PINS> = SpiRxDma<$SPIi, REMAP, PINS, $RCi>;
         pub type $txdma<REMAP, PINS> = SpiTxDma<$SPIi, REMAP, PINS, $TCi>;
         pub type $rxtxdma<REMAP, PINS> = SpiRxTxDma<$SPIi, REMAP, PINS, $RCi, $TCi>;
@@ -635,14 +625,14 @@ macro_rules! spi_dma {
 
         impl<B, REMAP, PIN> crate::dma::ReadDma<B, u8> for SpiRxDma<$SPIi, REMAP, PIN, $RCi>
         where
-            B: StaticWriteBuffer<Word = u8>,
+            B: WriteBuffer<Word = u8>,
         {
             fn read(mut self, mut buffer: B) -> Transfer<W, B, Self> {
                 // NOTE(unsafe) We own the buffer now and we won't call other `&mut` on it
                 // until the end of the transfer.
-                let (ptr, len) = unsafe { buffer.static_write_buffer() };
+                let (ptr, len) = unsafe { buffer.write_buffer() };
                 self.channel.set_peripheral_address(
-                    unsafe { &(*$SPIi::ptr()).dr as *const _ as u32 },
+                    unsafe { &(*<$SPIi>::ptr()).dr as *const _ as u32 },
                     false,
                 );
                 self.channel.set_memory_address(ptr as u32, true);
@@ -678,14 +668,14 @@ macro_rules! spi_dma {
 
         impl<B, REMAP, PIN> crate::dma::WriteDma<B, u8> for SpiTxDma<$SPIi, REMAP, PIN, $TCi>
         where
-            B: StaticReadBuffer<Word = u8>,
+            B: ReadBuffer<Word = u8>,
         {
             fn write(mut self, buffer: B) -> Transfer<R, B, Self> {
                 // NOTE(unsafe) We own the buffer now and we won't call other `&mut` on it
                 // until the end of the transfer.
-                let (ptr, len) = unsafe { buffer.static_read_buffer() };
+                let (ptr, len) = unsafe { buffer.read_buffer() };
                 self.channel.set_peripheral_address(
-                    unsafe { &(*$SPIi::ptr()).dr as *const _ as u32 },
+                    unsafe { &(*<$SPIi>::ptr()).dr as *const _ as u32 },
                     false,
                 );
                 self.channel.set_memory_address(ptr as u32, true);
@@ -722,8 +712,8 @@ macro_rules! spi_dma {
         impl<RXB, TXB, REMAP, PIN> crate::dma::ReadWriteDma<RXB, TXB, u8>
             for SpiRxTxDma<$SPIi, REMAP, PIN, $RCi, $TCi>
         where
-            RXB: StaticWriteBuffer<Word = u8>,
-            TXB: StaticReadBuffer<Word = u8>,
+            RXB: WriteBuffer<Word = u8>,
+            TXB: ReadBuffer<Word = u8>,
         {
             fn read_write(
                 mut self,
@@ -732,22 +722,22 @@ macro_rules! spi_dma {
             ) -> Transfer<W, (RXB, TXB), Self> {
                 // NOTE(unsafe) We own the buffer now and we won't call other `&mut` on it
                 // until the end of the transfer.
-                let (rxptr, rxlen) = unsafe { rxbuffer.static_write_buffer() };
-                let (txptr, txlen) = unsafe { txbuffer.static_read_buffer() };
+                let (rxptr, rxlen) = unsafe { rxbuffer.write_buffer() };
+                let (txptr, txlen) = unsafe { txbuffer.read_buffer() };
 
                 if rxlen != txlen {
                     panic!("receive and send buffer lengths do not match!");
                 }
 
                 self.rxchannel.set_peripheral_address(
-                    unsafe { &(*$SPIi::ptr()).dr as *const _ as u32 },
+                    unsafe { &(*<$SPIi>::ptr()).dr as *const _ as u32 },
                     false,
                 );
                 self.rxchannel.set_memory_address(rxptr as u32, true);
                 self.rxchannel.set_transfer_length(rxlen);
 
                 self.txchannel.set_peripheral_address(
-                    unsafe { &(*$SPIi::ptr()).dr as *const _ as u32 },
+                    unsafe { &(*<$SPIi>::ptr()).dr as *const _ as u32 },
                     false,
                 );
                 self.txchannel.set_memory_address(txptr as u32, true);
@@ -804,7 +794,28 @@ macro_rules! spi_dma {
     };
 }
 
-spi_dma!(SPI1, dma1::C2, dma1::C3, Spi1RxDma, Spi1TxDma, Spi1RxTxDma);
-spi_dma!(SPI2, dma1::C4, dma1::C5, Spi2RxDma, Spi2TxDma, Spi2RxTxDma);
+spi_dma!(
+    pac::SPI1,
+    dma1::C2,
+    dma1::C3,
+    Spi1RxDma,
+    Spi1TxDma,
+    Spi1RxTxDma
+);
+spi_dma!(
+    pac::SPI2,
+    dma1::C4,
+    dma1::C5,
+    Spi2RxDma,
+    Spi2TxDma,
+    Spi2RxTxDma
+);
 #[cfg(feature = "connectivity")]
-spi_dma!(SPI3, dma2::C1, dma2::C2, Spi3RxDma, Spi3TxDma, Spi3RxTxDma);
+spi_dma!(
+    pac::SPI3,
+    dma2::C1,
+    dma2::C2,
+    Spi3RxDma,
+    Spi3TxDma,
+    Spi3RxTxDma
+);
